@@ -18,7 +18,7 @@ pub use crate::{
 use crate::{
     accept_language::{LanguageTags, UserLanguage},
     css::{Origin, Stylesheet},
-    document::{Document, LoadOptions, NodeId, RenderingOptions},
+    document::{Document, LoadOptions, NodeId},
     dpi::Dpi,
     drawing_ctx::SvgNesting,
     error::InternalRenderingError,
@@ -79,9 +79,6 @@ impl From<InternalRenderingError> for RenderingError {
             InternalRenderingError::InvalidTransform => {
                 RenderingError::Rendering("invalid transform".to_string())
             }
-            InternalRenderingError::CircularReference(c) => {
-                RenderingError::Rendering(format!("circular reference in node {c}"))
-            }
             InternalRenderingError::IdNotFound => RenderingError::IdNotFound,
             InternalRenderingError::InvalidId(s) => RenderingError::InvalidId(s),
             InternalRenderingError::OutOfMemory(s) => RenderingError::OutOfMemory(s),
@@ -148,7 +145,7 @@ impl Loader {
     ///
     /// This is useful when a `Loader` must be created by the C API, which should already
     /// have created a session for logging.
-    #[cfg(feature = "c-api")]
+    #[cfg(feature = "capi")]
     pub fn new_with_session(session: Session) -> Self {
         Self {
             unlimited_size: false,
@@ -610,15 +607,6 @@ impl<'a> CairoRenderer<'a> {
         Some(self.width_height_to_user(self.dpi))
     }
 
-    fn rendering_options(&self) -> RenderingOptions {
-        RenderingOptions {
-            dpi: self.dpi,
-            user_language: self.user_language.clone(),
-            svg_nesting: SvgNesting::Standalone,
-            testing: self.is_testing,
-        }
-    }
-
     /// Renders the whole SVG document fitted to a viewport
     ///
     /// The `viewport` gives the position and size at which the whole SVG
@@ -636,7 +624,10 @@ impl<'a> CairoRenderer<'a> {
             &self.handle.session,
             cr,
             viewport,
-            &self.rendering_options(),
+            &self.user_language,
+            self.dpi,
+            SvgNesting::Standalone,
+            self.is_testing,
         )?)
     }
 
@@ -676,7 +667,9 @@ impl<'a> CairoRenderer<'a> {
             &self.handle.session,
             node,
             viewport,
-            &self.rendering_options(),
+            &self.user_language,
+            self.dpi,
+            self.is_testing,
         )?)
     }
 
@@ -713,7 +706,10 @@ impl<'a> CairoRenderer<'a> {
             cr,
             node,
             viewport,
-            &self.rendering_options(),
+            &self.user_language,
+            self.dpi,
+            SvgNesting::Standalone,
+            self.is_testing,
         )?)
     }
 
@@ -757,7 +753,9 @@ impl<'a> CairoRenderer<'a> {
         Ok(self.handle.document.get_geometry_for_element(
             &self.handle.session,
             node,
-            &self.rendering_options(),
+            &self.user_language,
+            self.dpi,
+            self.is_testing,
         )?)
     }
 
@@ -791,12 +789,14 @@ impl<'a> CairoRenderer<'a> {
             cr,
             node,
             element_viewport,
-            &self.rendering_options(),
+            &self.user_language,
+            self.dpi,
+            self.is_testing,
         )?)
     }
 
     #[doc(hidden)]
-    #[cfg(feature = "c-api")]
+    #[cfg(feature = "capi")]
     pub fn dpi(&self) -> Dpi {
         self.dpi
     }
@@ -824,7 +824,7 @@ impl<'a> CairoRenderer<'a> {
     }
 
     #[doc(hidden)]
-    #[cfg(feature = "c-api")]
+    #[cfg(feature = "capi")]
     pub fn test_mode(self, is_testing: bool) -> Self {
         CairoRenderer { is_testing, ..self }
     }
