@@ -18,16 +18,13 @@ use crate::bbox::BoundingBox;
 use crate::borrow_element_as;
 use crate::css::{self, Origin, Stylesheet};
 use crate::dpi::Dpi;
-use crate::drawing_ctx::{
-    draw_tree, with_saved_cr, DrawingMode, RenderingConfiguration, SvgNesting,
-};
+use crate::drawing_ctx::{draw_tree, with_saved_cr, DrawingMode, SvgNesting};
 use crate::error::{AcquireError, InternalRenderingError, LoadingError, NodeIdError};
 use crate::io::{self, BinaryData};
 use crate::is_element_of_type;
 use crate::limits;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeData};
 use crate::rect::Rect;
-use crate::rsvg_log;
 use crate::session::Session;
 use crate::structure::IntrinsicDimensions;
 use crate::surface_utils::shared_surface::SharedImageSurface;
@@ -126,30 +123,6 @@ impl LoadOptions {
     }
 }
 
-/// Document-level rendering options.
-///
-/// This gets then converted to a [`drawing_ctx::RenderingConfiguration`] when all the
-/// parameters are known.
-pub struct RenderingOptions {
-    pub dpi: Dpi,
-    pub user_language: UserLanguage,
-    pub svg_nesting: SvgNesting,
-    pub testing: bool,
-}
-
-impl RenderingOptions {
-    /// Copies the options to a [`RenderingConfiguration`], and adds the `measuring` flag.
-    fn to_rendering_configuration(&self, measuring: bool) -> RenderingConfiguration {
-        RenderingConfiguration {
-            dpi: self.dpi,
-            user_language: self.user_language.clone(),
-            svg_nesting: self.svg_nesting,
-            testing: self.testing,
-            measuring,
-        }
-    }
-}
-
 /// A loaded SVG file and its derived data.
 pub struct Document {
     /// Tree of nodes; the root is guaranteed to be an `<svg>` element.
@@ -229,6 +202,19 @@ impl Document {
         self.ids.get(id).map(|n| (*n).clone())
     }
 
+    /// Loads an image by URL, or returns a pre-loaded one.
+    fn lookup_image(&self, url: &str) -> Result<SharedImageSurface, LoadingError> {
+        let aurl = self
+            .load_options
+            .url_resolver
+            .resolve_href(url)
+            .map_err(|_| LoadingError::BadUrl)?;
+
+        self.resources
+            .borrow_mut()
+            .lookup_image(&self.session, &self.load_options, &aurl)
+    }
+
     /// Loads a resource by URL, or returns a pre-loaded one.
     fn lookup_resource(&self, url: &str) -> Result<Resource, LoadingError> {
         let aurl = self
@@ -283,10 +269,22 @@ impl Document {
         session: &Session,
         cr: &cairo::Context,
         viewport: &cairo::Rectangle,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        svg_nesting: SvgNesting,
+        is_testing: bool,
     ) -> Result<(), InternalRenderingError> {
         let root = self.root();
-        self.render_layer(session, cr, root, viewport, options)
+        self.render_layer(
+            session,
+            cr,
+            root,
+            viewport,
+            user_language,
+            dpi,
+            svg_nesting,
+            is_testing,
+        )
     }
 
     pub fn render_layer(
@@ -295,7 +293,10 @@ impl Document {
         cr: &cairo::Context,
         node: Node,
         viewport: &cairo::Rectangle,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        svg_nesting: SvgNesting,
+        is_testing: bool,
     ) -> Result<(), InternalRenderingError> {
         cr.status()?;
 
@@ -303,15 +304,17 @@ impl Document {
 
         let viewport = Rect::from(*viewport);
 
-        let config = options.to_rendering_configuration(false);
-
         with_saved_cr(cr, || {
             draw_tree(
                 session.clone(),
                 DrawingMode::LimitToStack { node, root },
                 cr,
                 viewport,
-                config,
+                user_language,
+                dpi,
+                svg_nesting,
+                false,
+                is_testing,
                 &mut AcquiredNodes::new(self),
             )
             .map(|_bbox| ())
@@ -323,21 +326,25 @@ impl Document {
         session: &Session,
         node: Node,
         viewport: Rect,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        is_testing: bool,
     ) -> Result<(Rect, Rect), InternalRenderingError> {
         let root = self.root();
 
         let target = cairo::ImageSurface::create(cairo::Format::Rgb24, 1, 1)?;
         let cr = cairo::Context::new(&target)?;
 
-        let config = options.to_rendering_configuration(true);
-
         let bbox = draw_tree(
             session.clone(),
             DrawingMode::LimitToStack { node, root },
             &cr,
             viewport,
-            config,
+            user_language,
+            dpi,
+            SvgNesting::Standalone,
+            true,
+            is_testing,
             &mut AcquiredNodes::new(self),
         )?;
 
@@ -352,11 +359,14 @@ impl Document {
         session: &Session,
         node: Node,
         viewport: &cairo::Rectangle,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        is_testing: bool,
     ) -> Result<(cairo::Rectangle, cairo::Rectangle), InternalRenderingError> {
         let viewport = Rect::from(*viewport);
 
-        let (ink_rect, logical_rect) = self.geometry_for_layer(session, node, viewport, options)?;
+        let (ink_rect, logical_rect) =
+            self.geometry_for_layer(session, node, viewport, user_language, dpi, is_testing)?;
 
         Ok((
             cairo::Rectangle::from(ink_rect),
@@ -368,21 +378,25 @@ impl Document {
         &self,
         session: &Session,
         node: &Node,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        is_testing: bool,
     ) -> Result<BoundingBox, InternalRenderingError> {
         let target = cairo::ImageSurface::create(cairo::Format::Rgb24, 1, 1)?;
         let cr = cairo::Context::new(&target)?;
 
         let node = node.clone();
 
-        let config = options.to_rendering_configuration(true);
-
         draw_tree(
             session.clone(),
             DrawingMode::OnlyNode(node),
             &cr,
             unit_rectangle(),
-            config,
+            user_language,
+            dpi,
+            SvgNesting::Standalone,
+            true,
+            is_testing,
             &mut AcquiredNodes::new(self),
         )
     }
@@ -392,9 +406,11 @@ impl Document {
         &self,
         session: &Session,
         node: Node,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        is_testing: bool,
     ) -> Result<(cairo::Rectangle, cairo::Rectangle), InternalRenderingError> {
-        let bbox = self.get_bbox_for_element(session, &node, options)?;
+        let bbox = self.get_bbox_for_element(session, &node, user_language, dpi, is_testing)?;
 
         let ink_rect = bbox.ink_rect.unwrap_or_default();
         let logical_rect = bbox.rect.unwrap_or_default();
@@ -414,11 +430,13 @@ impl Document {
         cr: &cairo::Context,
         node: Node,
         element_viewport: &cairo::Rectangle,
-        options: &RenderingOptions,
+        user_language: &UserLanguage,
+        dpi: Dpi,
+        is_testing: bool,
     ) -> Result<(), InternalRenderingError> {
         cr.status()?;
 
-        let bbox = self.get_bbox_for_element(session, &node, options)?;
+        let bbox = self.get_bbox_for_element(session, &node, user_language, dpi, is_testing)?;
 
         if bbox.ink_rect.is_none() || bbox.rect.is_none() {
             // Nothing to draw
@@ -441,14 +459,16 @@ impl Document {
             cr.scale(factor, factor);
             cr.translate(-ink_r.x0, -ink_r.y0);
 
-            let config = options.to_rendering_configuration(false);
-
             draw_tree(
                 session.clone(),
                 DrawingMode::OnlyNode(node),
                 cr,
                 unit_rectangle(),
-                config,
+                user_language,
+                dpi,
+                SvgNesting::Standalone,
+                false,
+                is_testing,
                 &mut AcquiredNodes::new(self),
             )
             .map(|_bbox| ())
@@ -516,6 +536,23 @@ impl Resources {
         }
     }
 
+    fn lookup_image(
+        &mut self,
+        session: &Session,
+        load_options: &LoadOptions,
+        aurl: &AllowedUrl,
+    ) -> Result<SharedImageSurface, LoadingError> {
+        // FIXME: pass a cancellable to this.  This function is called
+        // at rendering time, so probably the cancellable should come
+        // from cancellability in CairoRenderer - see #429
+        let resource = self.lookup_resource(session, load_options, aurl, None)?;
+
+        match resource {
+            Resource::Image(image) => Ok(image),
+            _ => Err(LoadingError::Other(format!("{aurl} is not a raster image"))),
+        }
+    }
+
     fn lookup_resource(
         &mut self,
         session: &Session,
@@ -602,6 +639,10 @@ fn image_format(content_type: &str) -> Result<image::ImageFormat, LoadingError>
         "image/jpeg" => Ok(image::ImageFormat::Jpeg),
         "image/gif" => Ok(image::ImageFormat::Gif),
         "image/webp" => Ok(image::ImageFormat::WebP),
+
+        #[cfg(feature = "avif")]
+        "image/avif" => Ok(image::ImageFormat::Avif),
+
         _ => Err(LoadingError::Other(format!(
             "unsupported image format {content_type}"
         ))),
@@ -731,7 +772,6 @@ pub struct AcquiredNodes<'i> {
     document: &'i Document,
     num_elements_acquired: usize,
     node_stack: Rc<RefCell<NodeStack>>,
-    nodes_with_cycles: Vec<Node>,
 }
 
 impl<'i> AcquiredNodes<'i> {
@@ -740,10 +780,13 @@ impl<'i> AcquiredNodes<'i> {
             document,
             num_elements_acquired: 0,
             node_stack: Rc::new(RefCell::new(NodeStack::new())),
-            nodes_with_cycles: Vec::new(),
         }
     }
 
+    pub fn lookup_image(&self, href: &str) -> Result<SharedImageSurface, LoadingError> {
+        self.document.lookup_image(href)
+    }
+
     pub fn lookup_resource(&self, url: &str) -> Result<Resource, LoadingError> {
         self.document.lookup_resource(url)
     }
@@ -773,10 +816,6 @@ impl<'i> AcquiredNodes<'i> {
             .lookup_node(node_id)
             .ok_or_else(|| AcquireError::LinkNotFound(node_id.clone()))?;
 
-        if self.nodes_with_cycles.contains(&node) {
-            return Err(AcquireError::CircularReference(node.clone()));
-        }
-
         if node.borrow_element().is_accessed_by_reference() {
             self.acquire_ref(&node)
         } else {
@@ -796,11 +835,8 @@ impl<'i> AcquiredNodes<'i> {
     /// * At the drawing stage, `acquire_ref()` the pattern node that we already had, so that
     /// its child elements that reference other paint servers will be able to detect circular
     /// references to the pattern.
-    pub fn acquire_ref(&mut self, node: &Node) -> Result<AcquiredNode, AcquireError> {
-        if self.nodes_with_cycles.contains(node) {
-            Err(AcquireError::CircularReference(node.clone()))
-        } else if self.node_stack.borrow().contains(node) {
-            self.nodes_with_cycles.push(node.clone());
+    pub fn acquire_ref(&self, node: &Node) -> Result<AcquiredNode, AcquireError> {
+        if self.node_stack.borrow().contains(node) {
             Err(AcquireError::CircularReference(node.clone()))
         } else {
             self.node_stack.borrow_mut().push(node);
@@ -903,15 +939,10 @@ impl DocumentBuilder {
         let node = Node::new(NodeData::new_element(&self.session, name, attrs));
 
         if let Some(id) = node.borrow_element().get_id() {
-            match self.ids.entry(id.to_string()) {
-                Entry::Occupied(_) => {
-                    rsvg_log!(self.session, "ignoring duplicate id {id} for {node}");
-                }
-
-                Entry::Vacant(e) => {
-                    e.insert(node.clone());
-                }
-            }
+            // This is so we don't overwrite an existing id
+            self.ids
+                .entry(id.to_string())
+                .or_insert_with(|| node.clone());
         }
 
         if let Some(parent) = parent {
